from flask import current_app, request
from flask_login import current_user
from socket import gaierror
from . import cqi
from .. import db, logger, socketio
from ..decorators import socketio_login_required
from ..events import connected_sessions
from ..models import Corpus, User
import math


'''
' A dictionary containing lists of, with corpus ids associated, Socket.IO
' session ids (sid). {<corpus_id>: [<sid>, ...], ...}
'''
corpus_analysis_sessions = {}
'''
' A dictionary containing Socket.IO session id - CQi client pairs.
' {<sid>: CQiClient, ...}
'''
corpus_analysis_clients = {}


@socketio.on('corpus_analysis_init')
@socketio_login_required
def init_corpus_analysis(corpus_id):
    socketio.start_background_task(corpus_analysis_session_handler,
                                   current_app._get_current_object(),
                                   corpus_id, current_user.id, request.sid)


@socketio.on('corpus_analysis_query')
@socketio_login_required
def corpus_analysis_query(query):
    client = corpus_analysis_clients.get(request.sid)
    if client is None:
        response = {'code': 424, 'desc': 'No client found for this session',
                    'msg': 'Failed Dependency'}
        socketio.emit('corpus_analysis_query', response, room=request.sid)
        return
    if client.status == 'running':
        client.status = 'abort'
        while client.status != 'ready':
            socketio.sleep(0.1)
    try:
        corpus = client.corpora.get('CORPUS')
        query_status = corpus.query(query)
        results = corpus.subcorpora.get('Results')
    except cqi.errors.CQiException as e:
        payload = {'code': e.code, 'desc': e.description, 'msg': e.name}
        response = {'code': 500, 'desc': None, 'msg': 'Internal Server Error',
                    'payload': payload}
        socketio.emit('corpus_analysis_query', response, room=request.sid)
        return
    payload = {**query_status, 'match_count': results.attrs['size']}
    response = {'code': 200, 'desc': None, 'msg': 'OK', 'payload': payload}
    socketio.emit('corpus_analysis_query', response, room=request.sid)
    # TODO: Stop here and add a new method for transmission
    chunk_size = 100
    chunk_start = 0
    context = 100
    progress = 0
    client.status = 'running'
    while chunk_start <= results.attrs['size']:
        if client.status == 'abort':
            break
        chunk = results.export(context=context, cutoff=chunk_size,
                               expand_lists=False, offset=chunk_start)
        chunk['cpos_ranges'] = True
        if (results.attrs['size'] == 0):
            progress = 100
        else:
            progress = ((chunk_start + chunk_size) / results.attrs['size']) * 100
            progress = min(100, int(math.ceil(progress)))
        response = {'code': 200, 'desc': None, 'msg': 'OK',
                    'payload': {'chunk': chunk, 'progress': progress}}
        socketio.emit('corpus_analysis_query_results', response,
                      room=request.sid)
        chunk_start += chunk_size
    client.status = 'ready'


@socketio.on('corpus_analysis_inspect_match')
@socketio_login_required
def corpus_analysis_inspect_match(payload):
    payload = payload["payload"]
    client = corpus_analysis_clients.get(request.sid)
    if client is None:
        response = {'code': 424, 'desc': 'No client found for this session',
                    'msg': 'Failed Dependency'}
        socketio.emit('corpus_analysis_inspect_match', response, room=request.sid)
        return
    try:
        corpus = client.corpora.get('CORPUS')
        s = corpus.structural_attributes.get('s')
        payload = s.export(payload['first_cpos'], payload['last_cpos'], context=10)
        payload['cpos_ranges'] = True
    except cqi.errors.CQiException as e:
        payload = {'code': e.code, 'desc': e.description, 'msg': e.name}
        response = {'code': 500, 'desc': None, 'msg': 'Internal Server Error',
                    'payload': payload}
    else:
        response = {'code': 200, 'desc': None, 'msg': 'OK', 'payload': payload}
    socketio.emit('corpus_analysis_inspect_match', response, room=request.sid)


def corpus_analysis_session_handler(app, corpus_id, user_id, session_id):
    with app.app_context():
        ''' Setup analysis session '''
        corpus = Corpus.query.get(corpus_id)
        user = User.query.get(user_id)
        if corpus is None:
            response = {'code': 404, 'desc': None, 'msg': 'Not Found'}
            socketio.emit('corpus_analysis_init', response, room=session_id)
            return
        elif not (corpus.creator == user or user.is_administrator()):
            response = {'code': 403, 'desc': None, 'msg': 'Forbidden'}
            socketio.emit('corpus_analysis_init', response, room=session_id)
            return
        elif corpus.status == 'unprepared':
            response = {'code': 424, 'desc': 'Corpus is not prepared',
                        'msg': 'Failed Dependency'}
            socketio.emit('corpus_analysis_init', response, room=request.sid)
            return
        while corpus.status != 'analysing':
            db.session.refresh(corpus)
            socketio.sleep(3)
        client = cqi.CQiClient('corpus_{}_analysis'.format(corpus_id))
        try:
            payload = client.connect()
        except cqi.errors.CQiException as e:
            payload = {'code': e.code, 'desc': e.description, 'msg': e.name}
            response = {'code': 500, 'desc': None,
                        'msg': 'Internal Server Error', 'payload': payload}
            socketio.emit('corpus_analysis_init', response, room=session_id)
            return
        except gaierror:
            response = {'code': 500, 'desc': None,
                        'msg': 'Internal Server Error'}
            socketio.emit('corpus_analysis_init', response, room=session_id)
            return
        corpus_analysis_clients[session_id] = client
        if corpus_id not in corpus_analysis_sessions:
            corpus_analysis_sessions[corpus_id] = [session_id]
        else:
            corpus_analysis_sessions[corpus_id].append(session_id)
        client.status = 'ready'
        response = {'code': 200, 'desc': None, 'msg': 'OK', 'payload': payload}
        socketio.emit('corpus_analysis_init', response, room=session_id)
        ''' Observe analysis session '''
        while session_id in connected_sessions:
            socketio.sleep(3)
        ''' Teardown analysis session '''
        if client.status == 'running':
            client.status = 'abort'
            while client.status != 'ready':
                socketio.sleep(0.1)
        try:
            client.disconnect()
        except cqi.errors.CQiException:
            pass
        corpus_analysis_clients.pop(session_id, None)
        corpus_analysis_sessions[corpus_id].remove(session_id)
        if not corpus_analysis_sessions[corpus_id]:
            corpus_analysis_sessions.pop(corpus_id, None)
            corpus.status = 'stop analysis'
            db.session.commit()
